from __future__ import division
import numpy as np
cimport numpy as np
cimport cython


@cython.boundscheck(False)
@cython.wraparound(False)
def fast_generate_rank_difference_matrix(np.ndarray[np.int32_t, ndim=2]lst,
                                         np.ndarray[np.float32_t, ndim=2] D):
    cdef unsigned int i, j, k;
    for i in xrange(D.shape[0]):
        for j in xrange(i, D.shape[0]):
            if i==j: continue;
            for k in xrange(lst.shape[1]):
                if lst[i, k]>lst[j, k]:
                    D[i, j]+=lst[i, k]-lst[j, k]
                else:
                    D[j, i]+=lst[j, k]-lst[i, k]


@cython.boundscheck(False)
@cython.wraparound(False)
def fast_generate_list_difference_matrix(np.ndarray[np.int32_t, ndim=2]lst,
                                         np.ndarray[np.float32_t, ndim=2] D):
    cdef unsigned int i, j, k;
    for i in xrange(D.shape[0]):
        for j in xrange(i, D.shape[0]):
            if i==j: continue;
            for k in xrange(lst.shape[1]):
                if lst[i, k]>lst[j, k]:
                    D[i, j]+=1
                else:
                    D[j, i]+=1



import pandas as pd
import numpy as np
from util import fast_convert
import sys


def user_rate_converter(filename, destfilename=None,
                        converttype='arithmetic_mean'):
    """This function is designed to convert a user_item_rate dataframe to
    item_pair_rate dataframe.
    There are altogether four convert types:
    'arithmetic_mean': one contest is the arithmetic mean of all watch records.
    'log_mean': one record is the log geometric mean of all watch records.
    'probability': compare by statistical frequency.
    """

    table = pd.read_hdf(filename, 'user_item_rate')
    table = table[['username', 'itemid', 'rate']]
    table.sort_values(by=['username', 'itemid'], inplace=True)
    iid = table.loc[:, 'itemid'].unique()
    idx = dict(zip(iid, range(iid.shape[0])))
    maxitem = len(idx)

    matx = np.zeros((maxitem, maxitem), dtype=np.float32)
    count = np.zeros((maxitem, maxitem), dtype=np.float32)

    b = 0
    for e in xrange(1, table.shape[0]+1):
        if e==table.shape[0] or table.iloc[e, 0]!=table.iloc[b, 0]: # new block start
            if e-b==1:
                b=e;
                continue;

            temptable = table.iloc[b:e]
            idxlst = np.zeros(e-b, dtype=np.int32)
            for i in xrange(e-b):
                idxlst[i] = idx[temptable.iloc[i, 1]]
            ratelst = np.require(temptable.iloc[:, 2].values, dtype=np.float32)
            if converttype=='arithmetic_mean':
                operation=1
            elif converttype=='log_mean':
                operation=2
            else:
                operation=3
            fast_convert(idxlst, ratelst, matx, count, operation)
            b=e
    count = np.triu(count+count.T, k=1)

    if destfilename is not None:
        # matrix is constructed. Now build the item pair table.
        primary = []
        secondary = []
        rate1 = []
        rate2 = []
        for i in xrange(maxitem):
            for j in xrange(i+1, maxitem):
                if count[i, j]!=0:
                    primary.append(iid[i])
                    secondary.append(iid[j])
                    rate1.append(matx[i, j]/count[i, j])
                    rate2.append(matx[j, i]/count[i, j])
        newtable = pd.DataFrame({'primary': primary,
                                 'secondary': secondary,
                                 'rate1': rate1,
                                 'rate2': rate2,
                                 'weight': np.ones(np.count_nonzero(count),
                                                   dtype=np.float32)},
                                 columns = ['primary', 'secondary', 'rate1',
                                            'rate2', 'weight'])
        newtable.to_hdf(destfilename, 'item_pair_rate')
    return (matx, count)

def run():
    while True:
        src = raw_input("Please input the source filename: ")
        if src: break
    dest = raw_input("Please input the dest filename (left black is the same as source file): ")
    if not dest: dest=src
    method = raw_input("Please input the matrix type: ")
    while True:
        if method !='arithmetic_mean' and method!='log_mean' and method!='probability':
            method = raw_input("Invalid matrix type. Please input the matrix type: ")
        else: break;
    user_rate_converter(src, dest, method)

if __name__ == '__main__':
    run()



from __future__ import division
import numpy as np
cimport numpy as np
cimport cython


@cython.boundscheck(False)
@cython.wraparound(False)
def fast_colley_matrix_build(np.ndarray[np.int32_t, ndim=2] pair,
                             np.ndarray[np.float32_t, ndim=2] rate,
                             np.ndarray[np.float32_t, ndim=2] C):
    cdef unsigned int i1, i2, i;
    cdef np.ndarray[np.float32_t, ndim=1] s = np.empty(C.shape[0], dtype=np.float32)
    for i in xrange(pair.shape[0]):
        i1 = <unsigned int>(pair[i, 0])
        i2 = <unsigned int>(pair[i, 1])
        C[i1, i2]-=rate[i, 2]
        C[i2, i1]-=rate[i, 2]
    s = -np.sum(C, axis=0)
    for i in xrange(C.shape[0]):
        C[i,i] = s[i]+2

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_colley_vector_build(np.ndarray[np.int32_t, ndim=2] pair,
                             np.ndarray[np.float32_t, ndim=2] rate,
                             np.ndarray[np.float32_t, ndim=1] b):
    cdef unsigned int i1, i2, i;
    for i in xrange(pair.shape[0]):
        i1 = <unsigned int>(pair[i, 0])
        i2 = <unsigned int>(pair[i, 1])
        if rate[i, 0]>rate[i, 1]:
            b[i1]+=rate[i, 2]
            b[i2]-=rate[i, 2]
        elif rate[i, 0]<rate[i, 1]:
            b[i1]-=rate[i, 2]
            b[i2]+=rate[i, 2]
        # tie wouldn't affect b
    b/=2
    b+=1

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_rate_diff_vote_matrix_build(np.ndarray[np.int32_t, ndim=2] pair,
                             np.ndarray[np.float32_t, ndim=2] rate,
                             np.ndarray[np.float32_t, ndim=2] D):
    cdef unsigned int i1, i2, i;
    for i in xrange(pair.shape[0]):
        i1 = <unsigned int>(pair[i, 0])
        i2 = <unsigned int>(pair[i, 1])
        if rate[i, 0]>rate[i, 1]:
            D[i2, i1] += (rate[i, 0]-rate[i, 1])*rate[i, 2]
        else:
            D[i1, i2] += (rate[i, 1]-rate[i, 0])*rate[i, 2]

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_simple_diff_vote_matrix_build(np.ndarray[np.int32_t, ndim=2] pair,
                             np.ndarray[np.float32_t, ndim=2] rate,
                             np.ndarray[np.float32_t, ndim=2] D):
    cdef unsigned int i1, i2, i;
    for i in xrange(pair.shape[0]):
        i1 = <unsigned int>(pair[i, 0])
        i2 = <unsigned int>(pair[i, 1])
        if rate[i, 0]>rate[i, 1]:
            D[i2, i1] += rate[i, 2]
        else:
            D[i1, i2] += rate[i, 2]

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_rate_vote_matrix_build(np.ndarray[np.int32_t, ndim=2] pair,
                             np.ndarray[np.float32_t, ndim=2] rate,
                             np.ndarray[np.float32_t, ndim=2] D):
    cdef unsigned int i1, i2, i;
    for i in xrange(pair.shape[0]):
        i1 = <unsigned int>(pair[i, 0])
        i2 = <unsigned int>(pair[i, 1])
        D[i2, i1] += rate[i, 0]*rate[i, 2]
        D[i1, i2] += rate[i, 1]*rate[i, 2]

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_consistancy_matrix_build(np.ndarray[np.float32_t, ndim=2] D,
                                  np.ndarray[np.float32_t, ndim=2] E,
                                  np.ndarray[np.int32_t, ndim=2] C):
    cdef unsigned int i, j, k;
    cdef int cnt, inv;
    for i in xrange(D.shape[0]):
        for j in xrange(i, D.shape[0]):
            if i==j: continue;
            cnt=0;inv=0;
            for k in xrange(D.shape[0]):
                if D[i, k]<D[j, k]:cnt+=1;
                elif D[i, k]>D[j, k]:inv+=1;
                if E[i, k]>E[j, k]:cnt+=1;
                elif E[i, k]<E[j, k]:inv+=1;
            C[i, j]=cnt;
            C[j, i]=inv;

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_contest_count_matrix_build(np.ndarray[np.int32_t, ndim=2] pair,
                                    np.ndarray[np.float32_t, ndim=2] C):
    cdef unsigned int i, ii, ij;
    for i in xrange(pair.shape[0]):
        ii = <unsigned int>(pair[i, 0])
        ij = <unsigned int>(pair[i, 1])
        C[ii, ij]+=1



import numpy as np
from BaseRank import BaseRank

class DifferenceRank(BaseRank):
    """docstring for DifferenceRank"""
    def __init__(self, *args, **kwargs):
        super(DifferenceRank, self).__init__(*args, **kwargs)

    def rate(self, K):
        return K.sum(axis=1)/K.shape[0]



import pandas as pd
import numpy as np


class BaseRank(object):
    """docstring for BaseRank"""

    def __init__(self, itemlist, ascending=False):
        """itemlist is a dataframe that contains two columns:
        `itemid` and `index`.
        """
        if itemlist is None:
            raise KeyError("There should be a solid item list for a rank task.")
        # The proper behaviour is to check if index is ascending.
        self.itemlist = itemlist
        self.ascending = ascending

    def rank(self, rate):
        itemlist = self.itemlist
        ascending = self.ascending
        assert (itemlist.shape[0] == len(rate))
        table = pd.DataFrame({
            'title': itemlist['itemid'],
            'rate': pd.Series(rate)
        }, columns = [['title', 'rate']])
        ranked = pd.DataFrame(table.sort_values(by='rate', ascending=ascending).\
                 values, columns = table.columns)
        # buggy
        #ranked['rank'] = pd.Series(range(1, len(rate) + 1), dtype='int32')
        ranked['rank'] = pd.Series(self._get_true_rank(ranked), dtype='int32')
        return ranked

    def _get_true_rank(self, rankedtable):
        # rankedtable: ['title', 'rate']
        r = np.zeros(rankedtable.shape[0], dtype = np.int32)
        r[0]=1;
        for i in xrange(1, rankedtable.shape[0]):
            if rankedtable.iloc[i, 1]!=rankedtable.iloc[i-1, 1]:
                r[i]=i+1
            else:
                r[i]=r[i-1]
        return r



from numpy.testing import assert_array_equal, assert_array_almost_equal
from numpy.testing import assert_almost_equal
from nose.tools import assert_raises, assert_true, assert_equal, assert_false
from rankit.ranker import BaseRank
import numpy as np
import pandas as pd


# toy testcase
sample_item_list = pd.DataFrame({'itemid':['A', 'B', 'C'],
                              'index':[2,1,0]},
                              columns=['itemid', 'index'])
sample_rate = np.array([1,1,2])

def same_rate_test():
    ranker = BaseRank(sample_item_list, ascending=True)
    ranktable = ranker.rank(sample_rate)
    ranktable = ranktable[['title', 'rate', 'rank']]
    assert_array_equal(ranktable.iloc[:, 2].values,
                       np.array([1,1,3], dtype = np.int32))



import numpy as np
from fast_converter import fast_consistancy_matrix_build

def ConsistancyMatrix(D):
    """
    For input D, each element indicates how much superior i is compared to j.
    c_ij = #{ k | d_ik < d_jk } + # how many contests j is superior to i
           #{ k | d_ki > d_kj }   # how many other items think that j is more hard to compete than i
    C_ij is the potential that j is superior to i.
    """
    C = np.zeros(D.shape, dtype=np.int32)
    D = np.require(D, dtype=np.float32)
    E = D.T;
    fast_consistancy_matrix_build(D, E, C)
    return C



import pandas as pd
import warnings
from rankit.ranker import BaseRank, MarkovRank, LeastViolatedRank
import numpy as np
from fast_list_matrix import fast_generate_rank_difference_matrix,\
fast_generate_list_difference_matrix


class RankManager(object):
    """
    Ranker Merger should manage different ranking results produced by ranker.
    """
    def __init__(self, availableranks=dict()):
        """
        availableranks: a dictionary with the method of ranking as key and
                        ranking result as value. Can be left blank.
        """
        super(RankManager, self).__init__()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            assert(type(availableranks)==dict)
        cnt = len(availableranks)
        self.cnt=0

        if cnt!=0:
            for k, v in availableranks.iteritems():
                assert(type(v)==pd.core.frame.DataFrame)
                v = v[['title', 'rate', 'rank']]
                if self.cnt==0:
                    ranktable = pd.DataFrame({'title': v.iloc[:, 0].values,
                                              k: v.iloc[:, 2].values},
                                             columns=['title', k])
                    self.cnt+=1
                else:
                    ranktable = ranktable.merge(v, how='outer', on='title')
                    ranktable.rename(columns={'rank':k}, inplace=True)
                    ranktable.drop('rate', axis=1, inplace=True, errors='ignore')
                    self.cnt+=1
            self.ranktable=ranktable
        else:
            self.ranktable=None

    def update(self, rankmethod, newrank):
        """
        rankmethod: the ranktable's ranking method. Should be unique against all
                    the available ranking methods.
        newrank:    the new ranktable.
        return:     None
        """
        newrank = newrank[['title', 'rate', 'rank']]
        ranktable = self.ranktable
        if self.cnt!=0:
            if rankmethod in ranktable.columns:
                raise KeyError("Duplicated rankmethod!")
            ranktable = ranktable.merge(newrank, how='outer', on='title')
            ranktable.rename(columns={'rank': rankmethod}, inplace=True)
            ranktable.drop('rate', axis=1, inplace=True, errors='ignore')
            self.ranktable = ranktable
            self.cnt+=1
        else:
            ranktable = pd.DataFrame({'title': newrank.iloc[:, 0].values,
                                      rankmethod: newrank.iloc[:, 2].values},
                                     columns=['title', rankmethod])
            self.ranktable = ranktable
            self.cnt+=1

    def delete(self, rankmethod, ignore=False):
        """
        rankmethod: the rankmethod's rank to be deleted.
        return:     None
        """
        ranktable = self.ranktable
        if not ignore:
            ranktable.drop(rankmethod, axis=1, inplace=True, errors='raise')
        else:
            ranktable.drop(rankmethod, axis=1, inplace=True, errors='ignore')
        self.ranktable = ranktable
        self.cnt-=1

    def get(self, rankmethod, default=None):
        """
        rankmethod: the rankmethod's rank to be retrived.
        default:    if the rank couldn't be retrived, the return value should be
                    default. If default is None, it will raise an exception!
        return:     dataframe
        """
        ranktable = self.ranktable
        try:
            rtn = ranktable[['title', rankmethod]]
            return rtn
        except KeyError as e:
            if default is None:
                raise e
            else:
                return default


class RankMerger(RankManager):
    def __init__(self, *args, **kwargs):
        super(RankMerger, self).__init__(*args, **kwargs)

    def BordaCountMerge(self):
        """
        return: a new DataFrame ranktable, with column=['title', 'rank']
        """
        if self.cnt==0:
            return pd.DataFrame(columns=['title', 'rate', 'rank'])
        ranktable = self.ranktable
        # ignore all the items that have nan as rank
        ranktable = ranktable.dropna()
        methods = ranktable.columns.drop('title')
        candidate_score = np.zeros((ranktable.shape[0], ranktable.shape[1]-1),
                                   np.int32)

        for c in xrange(methods.shape[0]):
            method = methods[c]
            tmprank = ranktable[['title', method]].sort_values(by=method)
            tmprank['_borda_score'] = pd.Series(
                self._get_borda_score(tmprank.loc[:,method].values),
                index=tmprank.index)
            tmprank.sort_index(inplace=True)
            candidate_score[:, c]=tmprank.loc[:,'_borda_score']
        borda_score = candidate_score.sum(axis=1)

        tmpitemlst = pd.DataFrame({'itemid': ranktable.loc[:,'title'],
                                   'index': range(ranktable.shape[0])},
                                  columns=['itemid', 'index'])
        ranker = BaseRank(tmpitemlst)
        rtn = ranker.rank(borda_score)
        return rtn

    def AverageRankMerge(self):
        if self.cnt==0:
            return pd.DataFrame(columns=['title', 'rate', 'rank'])
        ranktable = self.ranktable
        ranktable = ranktable.dropna()
        methods = ranktable.columns.drop('title')
        average_score = ranktable[methods].sum(axis=1).values

        tmpitemlst = pd.DataFrame({'itemid': ranktable.loc[:,'title'],
                                   'index': range(ranktable.shape[0])},
                                  columns=['itemid', 'index'])
        ranker = BaseRank(tmpitemlst, ascending=True)
        rtn = ranker.rank(average_score)
        return rtn

    def RankListVoteMerge(self, matrix_type="rankdifference", epsilon=0.75):
        """
        matrix_type: 'rankdifference' or 'listdifference'
        epsilon:     PR specific value
        """
        if self.cnt==0:
            return pd.DataFrame(columns=['title', 'rate', 'rank'])
        ranktable = self.ranktable
        ranktable = ranktable.dropna()
        methods = ranktable.columns.drop('title')
        D = np.zeros((ranktable.shape[0], ranktable.shape[0]), dtype=np.float32)
        if matrix_type=="rankdifference":
            fast_generate_rank_difference_matrix(ranktable[methods].values, D)
        elif matrix_type=="listdifference":
            fast_generate_list_difference_matrix(ranktable[methods].values, D)
        else:
            raise ValueError("Invalid matrix type.")

        tmpitemlst = pd.DataFrame({'itemid': ranktable.loc[:,'title'],
                                   'index': range(ranktable.shape[0])},
                                  columns=['itemid', 'index'])
        ranker = MarkovRank(itemlist=tmpitemlst, epsilon=epsilon)
        rate = ranker.rate(D)
        rtn = ranker.rank(rate)
        return rtn

    def LeastViolatedMerge(self, verbose=0):
        if self.cnt==0:
            return pd.DataFrame(columns=['title', 'rate', 'rank'])
        ranktable = self.ranktable
        ranktable = ranktable.dropna()
        methods = ranktable.columns.drop('title')
        C = np.zeros((ranktable.shape[0], ranktable.shape[0]), dtype=np.float32)
        fast_generate_list_difference_matrix(ranktable[methods].values, C)
        # originally in this funciton, C[i, j] is the list count that ranked i lower than j
        # but here, C[i, j] should be the number of lists that i is ranked higher than j minus
        # the number of lists that i is ranked lower than j.
        C = C.T-C

        tmpitemlst = pd.DataFrame({'itemid': ranktable.loc[:,'title'],
                                   'index': range(ranktable.shape[0])},
                                  columns=['itemid', 'index'])
        ranker = LeastViolatedRank(itemlist=tmpitemlst, minimize=False,
                                   verbose=verbose, ascending=True)
        rate = ranker.rate(C)
        rtn = ranker.rank(rate)
        return rtn


    def _get_borda_score(self, rate):
        score = np.zeros(rate.shape)
        # one thing that could be made sure is that rate is sorted.
        b=0
        for e in xrange(1, rate.shape[0]+1):
            if e==rate.shape[0] or rate[b]!=rate[e]:
                for i in xrange(b, e):
                    score[i] = rate.shape[0]-e
                b=e
        return score

class RankComparer(RankManager):
    def __init__(self, *args, **kwargs):
        super(RankComparer, self).__init__(*args, **kwargs)

    def KendallMeasure(self, method1, method2):
        """Kandall Measure ranges from -1 to 1
        The closer to 1, the more similar two ranks are.
        The closer to -1, the less similar two ranks are.
        Currently, this only applies to complete comparision.
        """
        ranktable = self.ranktable
        ranktable = ranktable.dropna() # Force Complete Kendall meaure
        methods = ranktable.columns.drop('title')
        a = ranktable[[method1, method2]].sort_values(by=method1).values
        nd=0.0;
        for i in xrange(a.shape[0]):
            for b in xrange(a.shape[0]-1, i, -1):
                if a[b-1, 1]>a[b, 1]:
                    a[b, 1], a[b-1, 1] = a[b-1, 1], a[b, 1]
                    nd+=1;
        n = a.shape[0]*(a.shape[0]-1)/2.0
        return (n-nd*2)/n

    def KendallMeasureMatrix(self):
        ranktable = self.ranktable
        ranktable = ranktable.dropna() # Force Complete Kendall meaure
        methods = ranktable.columns.drop('title')
        mtr = np.zeros((methods.shape[0], methods.shape[0]), dtype=np.float)
        for i in xrange(methods.shape[0]):
            for j in xrange(i, methods.shape[0]):
                if i==j: mtr[i, j]=1.0;
                else:
                    mtr[i, j]=self.KendallMeasure(methods[i], methods[j])
                    mtr[j, i]=mtr[i, j]
        return mtr;

    def SpearmanMeasure(self, method1, method2):
        """SpearmanMeaure ranges from 0 to infinite.
        The smaller Spearman Measure, the more similar to ranks are.
        """
        ranktable = self.ranktable
        ranktable = ranktable.dropna() # Force Complete Kendall meaure
        methods = ranktable.columns.drop('title')
        a = ranktable[[method1, method2]].values
        phi = 0.0
        for k in xrange(a.shape[0]):
            phi+=abs(a[k, 0] - a[k, 1])/float(min(a[k, 0], a[k, 1]))
        return phi

    def SpearmanMeaureMatrix(self):
        ranktable = self.ranktable
        ranktable = ranktable.dropna() # Force Complete Kendall meaure
        methods = ranktable.columns.drop('title')
        mtr = np.zeros((methods.shape[0], methods.shape[0]), dtype=np.float)
        for i in xrange(methods.shape[0]):
            for j in xrange(i, methods.shape[0]):
                if i==j: mtr[i, j]=0.0;
                else:
                    mtr[i, j]=self.SpearmanMeasure(methods[i], methods[j])
                    mtr[j, i]=mtr[i, j]
        return mtr;



from rankit.solver import DefaultSolver, InsufficientRankSolver
from BaseRank import BaseRank


class ColleyRank(BaseRank):
    """docstring for ColleyRank"""

    def __init__(self, *args, **kwargs):
        self.solver = DefaultSolver()
        super(ColleyRank, self).__init__(*args, **kwargs)

    def rate(self, C, b):
        solver = self.solver
        return solver.solve(C, b)


class MasseyRank(BaseRank):
    def __init__(self, *args, **kwargs):
        self.solver = InsufficientRankSolver(force=1)
        super(MasseyRank, self).__init__(*args, **kwargs)

    def rate(self, M, b):
        solver = self.solver
        return solver.solve(M, b)



from rankit.util import Converter
from rankit.ranker import *
from rankit.manager import *
import cPickle


def main():
    ranks = dict()
    cvt = Converter(filename='arithmetic_average.bin')

    C=cvt.ColleyMatrix()
    b=cvt.ColleyVector()
    ranker=ColleyRank(itemlist=cvt.ItemList())
    x=ranker.rate(C,b)
    r=ranker.rank(x)
    ranks['ariavg_colley']=r

    M=cvt.MasseyMatrix()
    b=cvt.MasseyVector()
    ranker=MasseyRank(itemlist=cvt.ItemList())
    x=ranker.rate(M,b)
    r=ranker.rank(x)
    ranks['ariavg_massey']=r

    M=cvt.SymmetricDifferenceMatrix()
    ranker=DifferenceRank(itemlist=cvt.ItemList())
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['ariavg_differ']=r

    M=cvt.RateVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['ariavg_markov_rv']=r

    M=cvt.RateDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['ariavg_markov_rdv']=r

    M=cvt.SimpleDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['ariavg_markov_sdv']=r

    M=cvt.RateVoteMatrix()
    ranker=ODRank(itemlist=cvt.ItemList(), iteration=1000000)
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['ariavg_od']=r

    C = cvt.CountMatrix()
    ranker=KeenerRank(itemlist=cvt.ItemList(), epsilon=1e-4)
    x = ranker.rate(M.T, C)
    r = ranker.rank(x)
    ranks['ariavg_keener']=r

    cvt = Converter(filename='log_geo_average.bin')

    C=cvt.ColleyMatrix()
    b=cvt.ColleyVector()
    ranker=ColleyRank(itemlist=cvt.ItemList())
    x=ranker.rate(C,b)
    r=ranker.rank(x)
    ranks['geoavg_colley']=r

    M=cvt.MasseyMatrix()
    b=cvt.MasseyVector()
    ranker=MasseyRank(itemlist=cvt.ItemList())
    x=ranker.rate(M,b)
    r=ranker.rank(x)
    ranks['geoavg_massey']=r

    M=cvt.SymmetricDifferenceMatrix()
    ranker=DifferenceRank(itemlist=cvt.ItemList())
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['geoavg_differ']=r

    M=cvt.RateVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['geoavg_markov_rv']=r

    M=cvt.RateDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['geoavg_markov_rdv']=r

    M=cvt.SimpleDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['geoavg_markov_sdv']=r

    M=cvt.RateVoteMatrix()
    ranker=ODRank(itemlist=cvt.ItemList(), iteration=1000000)
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['geoavg_od']=r

    C = cvt.CountMatrix()
    ranker=KeenerRank(itemlist=cvt.ItemList(), epsilon=1e-4)
    x = ranker.rate(M.T, C)
    r = ranker.rank(x)
    ranks['geoavg_keener']=r

    cvt = Converter(filename='prob.bin')

    C=cvt.ColleyMatrix()
    b=cvt.ColleyVector()
    ranker=ColleyRank(itemlist=cvt.ItemList())
    x=ranker.rate(C,b)
    r=ranker.rank(x)
    ranks['prob_colley']=r

    M=cvt.MasseyMatrix()
    b=cvt.MasseyVector()
    ranker=MasseyRank(itemlist=cvt.ItemList())
    x=ranker.rate(M,b)
    r=ranker.rank(x)
    ranks['prob_massey']=r

    M=cvt.SymmetricDifferenceMatrix()
    ranker=DifferenceRank(itemlist=cvt.ItemList())
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['prob_differ']=r

    M=cvt.RateVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['prob_markov_rv']=r

    M=cvt.RateDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['prob_markov_rdv']=r

    M=cvt.SimpleDifferenceVoteMatrix()
    ranker=MarkovRank(itemlist=cvt.ItemList(), epsilon=0.6)
    x = ranker.rate(M)
    r = ranker.rank(x)
    ranks['prob_markov_sdv']=r

    M=cvt.RateVoteMatrix()
    ranker=ODRank(itemlist=cvt.ItemList(), iteration=1000000)
    x=ranker.rate(M)
    r=ranker.rank(x)
    ranks['prob_od']=r

    C = cvt.CountMatrix()
    ranker=KeenerRank(itemlist=cvt.ItemList(), epsilon=1e-4)
    x = ranker.rate(M.T, C)
    r = ranker.rank(x)
    ranks['prob_keener']=r

    mgr = RankMerger(availableranks=ranks)
    finalrank = mgr.BordaCountMerge()

    with open('result.pkl', 'wb') as fw:
        cPickle.dump(mgr, fw)
        cPickle.dump(finalrank, fw)

if __name__=="__main__":
    main()



from numpy.linalg import solve, matrix_rank
import numpy as np
from numpy.linalg import LinAlgError

class DefaultSolver(object):
    """docstring for DefaultSolver"""
    def __init__(self):
        pass

    def solve(self, A, b):
        return solve(A, b)

class InsufficientRankSolver(DefaultSolver):
    """This Solver provides several ways to handle the solving of a sigular
    linear equation.
    There are two options to solve a singular linear equation:
    epsilon:    by adding matrix A with an all-one matrix (will neglect diagonal
                values whenever A has all-zeros in main diagonal.)
    force:      by forcing the last row of A to all one and last value of b to 0.
    """
    def __init__(self, epsilon=None, force=None):
        self.epsilon = epsilon
        self.force = force

    def solve(self, A, b):
        epsilon = self.epsilon
        force = self.force
        while True:
            try:
                rtn = super(InsufficientRankSolver, self).solve(A, b)
                break;
            except LinAlgError as e:
                if epsilon is None and force is None:
                    raise RuntimeError("Did not provide a way to resolve sigular matrix")

            if epsilon is not None:
                diagval = np.sum(np.diagonal(A))
                E = np.ones(A.shape, A.dtype)*epsilon
                if diagval==0:
                    E-=np.diag(np.diag(E))
                A+=E
                epsilon = None
            else:
                A[-1,:] = np.ones(A.shape[0], A.dtype)
                b[-1] = 0
                force = None

        return rtn;



from numpy.testing import assert_array_equal, assert_array_almost_equal
from numpy.testing import assert_almost_equal
from nose.tools import assert_raises, assert_true, assert_equal, assert_false
from rankit.util import ConsistancyMatrix, Converter
import numpy as np
import pandas as pd


# toy test case
D = np.array([
        [0,1,0],
        [0,0,1],
        [2,0,0]
    ])
C = np.array([
        [0,2,2],
        [2,0,2],
        [2,2,0]
    ], np.int32)

testtable = pd.DataFrame({'primary':['A','B','C','A'],
                          'secondary':['B','C','B','C'],
                          'rate1':[1,2,1,2],
                          'rate2':[2,3,3,1],
                          'weight':[2,1,1,1]})

def consistancy_matrix_builder_test():
    assert_array_equal(ConsistancyMatrix(D), C)

def converter_item_list_test():
    cvt = Converter(table=testtable)
    assert_equal(cvt.ItemList().shape[0],3)
    assert_equal(cvt.itemlist['A'], 0)
    assert_equal(cvt.itemlist['B'], 1)
    assert_equal(cvt.itemlist['C'], 2)

def colley_test():
    cvt = Converter(table=testtable)
    assert_array_equal(cvt.ColleyMatrix(),
    np.array([[5,-2,-1],[-2,6,-2],[-1,-2,5]],dtype=np.float32))
    assert_array_equal(cvt.ColleyVector(),
    np.array([0.5, 2, 0.5], dtype=np.float32))

def massey_test():
    cvt = Converter(table=testtable)
    assert_array_equal(cvt.MasseyMatrix(),
    np.array([[3,-2,-1],[-2,4,-2],[-1,-2,3]],dtype=np.float32))
    assert_array_equal(cvt.MasseyVector(),
    np.array([-1, 3, -2], dtype=np.float32))

def count_test():
    cvt = Converter(table=testtable)
    assert_array_equal(cvt.CountMatrix(),
    np.array([[0,1,1],[0,0,1],[0,1,0]],dtype=np.float32))

def symmetric_difference_test():
    cvt = Converter(table=testtable)
    assert_array_equal(cvt.SymmetricDifferenceMatrix(),
    np.array([[0,-2,1],[2,0,0.5],[-1,-0.5,0]], dtype=np.float32))

def vote_matrix_test():
    cvt = Converter(table=testtable)
    assert_array_equal(cvt.SimpleDifferenceVoteMatrix(),
    np.array([[0,2,0],[0,0,1],[1,1,0]], dtype=np.float32))
    assert_array_equal(cvt.RateDifferenceVoteMatrix(),
    np.array([[0,2,0],[0,0,1],[1,2,0]], dtype=np.float32))
    assert_array_equal(cvt.RateVoteMatrix(),
    np.array([[0,4,1],[2,0,4],[2,5,0]], dtype=np.float32))



from distutils.core import setup
from Cython.Build import cythonize
from distutils.extension import Extension
import numpy

ext_modules=[
    Extension("util",
              sources=["util.pyx"],
              include_dirs=[numpy.get_include()],
              libraries=["m"] # Unix-like specific
    )
]

setup(
    ext_modules = cythonize(ext_modules)
)



from __future__ import division
import numpy as np
cimport numpy as np
ctypedef np.int32_t INDEXTYPE_t
ctypedef np.float32_t RATETYPE_t
from libc.math cimport log
cimport cython

@cython.boundscheck(False)
@cython.wraparound(False)
def fast_convert(np.ndarray[INDEXTYPE_t, ndim=1] idxlst,
                 np.ndarray[RATETYPE_t, ndim=1] ratelst,
                 np.ndarray[RATETYPE_t, ndim=2] matx,
                 np.ndarray[RATETYPE_t, ndim=2] count,
                 int operation):
    cdef unsigned int i, j;
    cdef unsigned int ii, ij;
    cdef unsigned int len = idxlst.shape[0];
    for i in xrange(len):
        for j in xrange(i+1, len):
            ii = <unsigned int>(idxlst[i]);
            ij = <unsigned int>(idxlst[j]);
            count[ii, ij]+=1
            if operation==1:
                matx[ii, ij]+=ratelst[i]
                matx[ij, ii]+=ratelst[j]
            elif operation==2:
                matx[ii, ij]+=log(ratelst[i])
                matx[ij, ii]+=log(ratelst[j])
            else:
                matx[ii, ij]+=1 if ratelst[i]>ratelst[j] else 0
                matx[ij, ii]+=1 if ratelst[i]<ratelst[j] else 0



import numpy as np
from BaseRank import BaseRank
from numpy.linalg import norm


class MarkovRank(BaseRank):
    """docstring for MarkovRank
    Please be noted that there are three options to force the forming of a
    irreducible matrix.
    equalvote will make a node with no out-link vote equally to all the nodes in
    a graph.
    epsilon: common practice in PageRank, S = epsilon*S+(1-epsilon)*E/n
    bounce: a node without out-link will bounce to every node that links to it.
    """
    def __init__(self, threshold=1e-4, equalvote=None, epsilon=None,
                 bounce=None, *args, **kwargs):
        super(MarkovRank, self).__init__(*args, **kwargs)
        self.equalvote = equalvote
        self.epsilon = epsilon
        self.bounce = bounce
        self.threshold = threshold

    def rate(self, S):
        l = S.shape[0]
        s = S.sum(axis=1)
        idx = np.where(s==0)[0]
        S = S/np.expand_dims(s, axis=1)
        S[idx,:]=0
        equalvote = self.equalvote
        epsilon = self.epsilon
        bounce = self.bounce
        if equalvote is not None:
            S[idx,:]=1./l
        elif epsilon is not None:
            S = epsilon*S+(1-epsilon)*np.ones((l, l), dtype = S.dtype)/float(l)
        elif bounce is not None:
            mask = np.require((S[:, idx]!=0).T, dtype=np.float)
            for i in xrange(idx.shape[0]):
                S[idx[i],:] = mask[i]
        else:
            if idx.shape[0]!=0:
                raise RuntimeError("Did not specify a method to make matrix irreducible.")

        threshold = self.threshold
        assert(np.any(np.isnan(S))==False)
        S = np.require(S.T, dtype=np.float32)
        x=np.ones(l,dtype=np.float32)
        prev = x.copy()
        while True:
            x/=x.sum()
            if norm(prev-x)<threshold:
                break;
            prev=x.copy()
            x=np.dot(S, x)
        return x

class ODRank(BaseRank):
    """Three options are given for output rank.
    'offence'
    'defence'
    'summary'
    """
    def __init__(self, threshold = 1e-4, output='summary',
                 epsilon=None, iteration=10000, *args, **kwargs):
        super(ODRank, self).__init__(*args, **kwargs)
        if output!='summary' and output!='defence' and output!='offence':
            raise ValueError("Wrong output rating.")
        self.output = output
        self.epsilon = epsilon
        self.threshold = threshold
        self.iteration = iteration

    def rate(self, A):
        output = self.output
        epsilon = self.epsilon
        threshold = self.threshold
        iteration = self.iteration
        if epsilon is not None:
            E = np.ones(A.shape, A.dtype)*epsilon
            E-=np.diag(np.diag(E))
            A+=E
        d = np.ones(A.shape[0], dtype=np.float32)
        d_prev = d.copy()
        cnt=0
        while True:
            d = np.dot(A, 1./np.dot(A.T, 1./d))
            if norm(d_prev-d)<threshold:
                break;
            else:
                cnt+=1;
                if cnt>iteration: break;
            d_prev = d.copy()

        o = np.dot(A.T, 1./d)
        if output=='summary':
            return o/d
        elif output=='offence':
            return o
        elif output=='defence':
            return d

class KeenerRank(BaseRank):
    """docstring for KeenerRank"""
    def __init__(self, regularization=True, func=None, epsilon=None, threshold=1e-4,
                 *args, **kwargs):
        super(KeenerRank, self).__init__(*args, **kwargs)
        self.regularization = regularization
        self.func = func
        self.epsilon = epsilon
        self.threshold = threshold

    def rate(self, A, C):
        regularization = self.regularization
        func = self.func
        epsilon = self.epsilon
        threshold = self.threshold
        B = self._KeenerMatrix(A, C, regularization, func, epsilon)
        prev_r = np.ones(B.shape[0])
        r = prev_r.copy()
        while True:
            r = r/np.sum(r)
            if norm(r-prev_r)<threshold:
                break;
            else:
                prev_r = r.copy()
            r = np.dot(B, r)
        return r

    def _KeenerMatrix(self, A, C, regularization, func, epsilon):
        """func is a regularization function imposed on every element of matrix.
        """
        # Apply Laplace Law
        B = A+A.T+2;
        A = A+1
        A = A/B
        # Regularization
        if func is not None:
            h = np.frompyfunc(func, 1, 1)
            A = np.require(h(A), dtype=np.float32)
        # divide by contest number
        C = C+C.T
        c = np.sum(C, axis=1)
        if regularization:
            A = A/np.expand_dims(c, axis=1)
        A[C==0]=0
        if epsilon is not None:
            A += epsilon*np.ones(A.shape, A.dtype)
        return A



import numpy as np
from BaseRank import BaseRank


class LeastViolatedRank(BaseRank):
    """
    A ranking method which does not utilize the concept of state.
    Equivalent to linear ordering problem, which can be solved by ILP.
    Perhaps very time consuming when number of ranked objects is large. Take care!!
    """
    def __init__(self, minimize=True, verbose=0, solvertp='SCIP', *args, **kwargs):
        """Two options for solvertp:
        'SCIP': ILP backended by http://scip.zib.de/
        'CBC': ILP backedded by https://projects.coin-or.org/Cbc
        """
        super(LeastViolatedRank, self).__init__(*args, **kwargs)
        if solvertp!='SCIP' and solvertp!='CBC':
            raise KeyError("Invalid solver option.")
        self.solvertp = solvertp
        self.minimize = minimize
        self.verbose = verbose

    def rate(self, C):
        """
        C: a potential matrix generated by rankit.util.ConsistancyMatrix
        """
        minimize = self.minimize
        verbose = self.verbose
        solvertp = self.solvertp
        from ortools.linear_solver import pywraplp
        if solvertp=='SCIP':
            solver = pywraplp.Solver('BILP',pywraplp.Solver.SCIP_MIXED_INTEGER_PROGRAMMING)
        else:
            solver = pywraplp.Solver('BILP',pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING)

        X=[]
        for i in xrange(C.shape[0]):
            temp = []
            for j in xrange(C.shape[1]):
                temp.append(solver.IntVar(0, solver.infinity(), "x_"+str(i)+"_"+str(j)))
            X.append(temp)


        constraint1=[]
        for i in xrange(C.shape[0]):
            for j in xrange(i, C.shape[1]):
                if i==j: continue;
                constraint1.append(solver.Constraint(1, 1))
                constraint1[-1].SetCoefficient(X[i][j],1)
                constraint1[-1].SetCoefficient(X[j][i],1)

        objective = solver.Objective()
        for i in xrange(C.shape[0]):
            for j in xrange(C.shape[0]):
                if i!=j:
                    objective.SetCoefficient(X[i][j], float(C[i,j]))

        if minimize:
            objective.SetMinimization()
        else:
            objective.SetMaximization()
        status = solver.Solve()
        if status!=0:
            raise RuntimeError("IP solver failed with error code %s"%str(status))
        x=np.fromfunction(np.vectorize(lambda i,j: X[int(i)][int(j)].solution_value()), C.shape, dtype=np.int32)

        niter = 1;
        while True:
            constraint2 = []
            nz = np.vstack(np.nonzero(x))
            for idx in xrange(nz.shape[1]):
                i = nz[0][idx]
                j = nz[1][idx]
                knz = np.nonzero(np.multiply(x[j,:], x[:,i]))[0]
                for k in knz:
                    # add triangle constraint!
                    if i==j or j==k or k==i: continue;
                    constraint2.append(solver.Constraint(-solver.infinity(), 2))
                    constraint2[-1].SetCoefficient(X[i][j], 1)
                    constraint2[-1].SetCoefficient(X[j][k], 1)
                    constraint2[-1].SetCoefficient(X[k][i], 1)
            if len(constraint2)==0:
                if verbose!=0:
                    print "Iteration #%s: %s new triangle constraints added. Problem solved!"\
                    %(str(niter), str(len(constraint2)))
                break;
            else:
                if verbose!=0:
                    print "Iteration #%s: %s new triangle constraints added."\
                    %(str(niter), str(len(constraint2)))
            status = solver.Solve()
            if status!=0:
                raise RuntimeError("IP solver failed with error code %s"%str(status))
            x=np.fromfunction(np.vectorize(lambda i,j: X[int(i)][int(j)].solution_value()), C.shape, dtype=np.int32)
            niter+=1
        return np.sum(x, axis=0)



import pandas as pd
import numpy as np
from fast_converter import fast_colley_matrix_build, fast_colley_vector_build,\
fast_rate_diff_vote_matrix_build, fast_simple_diff_vote_matrix_build, \
fast_rate_vote_matrix_build, fast_contest_count_matrix_build
from scipy.sparse import coo_matrix, csr_matrix

def pref_func(x):
    return 0.5+np.sign(a-0.5)*(np.sqrt(np.abs(2*a-1)))/2

class Converter(object):
    """Converter accepts a pandas DataFrame with the columns 'primary',
    'secondary', 'rate1', 'rate2' and 'weight'. The DataFrame can either be
    passed directly through table parameter or a hdf5 filename, in which the
    DataFrame must be indexed by 'item_pair_rate'.
    Users can call different matrix generator to generate their matrix wanted.
    """

    def __init__(self, table=None, filename=''):
        """
        table:      the pandas DataFrame that records rankable objects competition
                    record
        filename:   the hdf5 filename that stores the DataFrame. The DataFrame
                    must be indexed by 'item_pair_rate'.
        """
        if table is None:
            table = pd.read_hdf(filename, "item_pair_rate")
        table = table[['primary','secondary','rate1','rate2','weight']]
        self.table = table
        # itemid to index table
        idx = self._extract_list(self.table)
        self.itemlist = idx
        temptable = table.iloc[:,:2].values
        pair = np.fromfunction(np.vectorize(lambda i, j: idx[temptable[i,j]]),
                        temptable.shape)
        pair = np.require(pair, dtype=np.int32)
        self.pair = pair

    def _extract_list(self, table):
        iid = np.hstack([table.loc[:,'primary'].values,
              table.loc[:,'secondary'].values])
        iid = np.unique(iid)
        # index to itemid
        self.iid = iid
        item_id = dict(zip(iid, range(iid.shape[0])))
        return item_id

    def ItemList(self):
        table = self.itemlist
        itemlist =  pd.DataFrame({'itemid': table.keys(),
                             'index': table.values()},
                            columns=['itemid', 'index'])
        itemlist.sort_values(by='index',inplace=True)
        return pd.DataFrame(itemlist.values,
                            index = pd.Index(range(itemlist.shape[0])),
                            columns=['itemid', 'index'])

    def ColleyMatrix(self):
        idx = self.itemlist
        table = self.table

        icnt = len(idx)
        # allocate space for computing
        C = np.zeros((icnt, icnt), dtype=np.float32)
        pair = self.pair

        fast_colley_matrix_build(pair,
                        np.require(table.iloc[:,2:].values, dtype=np.float32),
                        C)

        return C

    def ColleyVector(self):
        idx = self.itemlist
        table = self.table

        icnt = len(idx)
        # allocate space for computing
        b = np.zeros(icnt, dtype=np.float32)
        pair = self.pair

        fast_colley_vector_build(pair,
                        np.require(table.iloc[:,2:].values, dtype=np.float32),
                        b)

        return b


    def RateDifferenceVoteMatrix(self):
        """This function outputs only Point Difference Matrix.
        It can be ensured that every element of the matrix are not less than 0
        """
        idx = self.itemlist
        table = self.table

        icnt = len(idx)
        # allocate space for computing
        D = np.zeros((icnt, icnt), dtype=np.float32)
        pair = self.pair

        fast_rate_diff_vote_matrix_build(pair,
                        np.require(table.iloc[:,2:].values, dtype=np.float32),
                        D)

        return D

    def SimpleDifferenceVoteMatrix(self):
        """This function outputs only Simple Difference Vote Matrix.
        """
        idx = self.itemlist
        table = self.table

        icnt = len(idx)
        # allocate space for computing
        D = np.zeros((icnt, icnt), dtype=np.float32)
        pair = self.pair

        fast_simple_diff_vote_matrix_build(pair,
                        np.require(table.iloc[:,2:].values, dtype=np.float32),
                        D)

        return D

    def RateVoteMatrix(self):
        """This function outputs only Simple Difference Vote Matrix.
        """
        idx = self.itemlist
        table = self.table

        icnt = len(idx)
        # allocate space for computing
        D = np.zeros((icnt, icnt), dtype=np.float32)
        pair = self.pair

        fast_rate_vote_matrix_build(pair,
                        np.require(table.iloc[:,2:].values, dtype=np.float32),
                        D)

        return D


    def MasseyMatrix(self):
        """This function produces X'WX
        """
        idx = self.itemlist
        table = self.table
        pair = self.pair
        j = np.ravel(pair)
        i = np.repeat(np.arange(table.shape[0], dtype=np.int32), 2, axis=0)
        data = np.array([[1,-1]],dtype=np.float32)
        data = np.ravel(np.repeat(data, table.shape[0], axis=0))
        X = coo_matrix((data, (i, j)), shape=(table.shape[0], len(idx)))
        X = X.tocsr()
        W = np.require(table.iloc[:,4].values, np.float32)
        W = coo_matrix((W, (np.arange(W.shape[0]), np.arange(W.shape[0])))).tocsr()
        return np.asarray((X.T*W*X).todense())


    def MasseyVector(self):
        """This function produces X'Wy
        """
        idx = self.itemlist
        table = self.table
        pair = self.pair
        j = np.ravel(pair)
        i = np.repeat(np.arange(table.shape[0], dtype=np.int32), 2, axis=0)
        data = np.array([[1,-1]],dtype=np.float32)
        data = np.ravel(np.repeat(data, table.shape[0], axis=0))
        X = coo_matrix((data, (i, j)), shape=(table.shape[0], len(idx)))
        X = X.tocsr()
        W = np.require(table.iloc[:,4].values, np.float32)
        y = table.iloc[:, 2].values - table.iloc[:, 3].values;
        Wy=np.multiply(W, y)
        return X.T*Wy

    def DataDifferenceMatrix(self):
        return self.RateDifferenceVoteMatrix().T

    def CountMatrix(self):
        """This matrix counts all the contests that have been recorded by the
        item_pair_rate table.
        If two teams held more than one contest, and they have different order, the matrix
        will also record them accordingly in the symmetric elements of the matrix.
        """
        idx = self.itemlist
        pair = self.pair

        icnt = len(idx)
        D = np.zeros((icnt, icnt), dtype=np.float32)
        fast_contest_count_matrix_build(pair, D)
        return D

    def SymmetricDifferenceMatrix(self):
        C = self.CountMatrix()
        D = self.DataDifferenceMatrix()
        C = np.triu(C+C.T)
        D = np.triu(D-D.T)
        S = D/C
        S[np.isnan(S)]=0;
        return S-S.T



from numpy.testing import assert_array_equal, assert_array_almost_equal
from numpy.testing import assert_almost_equal
from nose.tools import assert_raises, assert_true, assert_equal, assert_false
from rankit.solver import DefaultSolver, InsufficientRankSolver
import numpy as np


# toy test case
A = np.array([
    [1, 1],
    [1, 1]
], dtype=np.float32)
b = np.array([1, 1], dtype=np.float32)

def singular_matrix_test():
    solver = InsufficientRankSolver()
    assert_raises(RuntimeError, solver.solve, A, b)

def normal_matrix_test():
    solver = InsufficientRankSolver()
    x = solver.solve(np.eye(2), b)
    assert_array_equal(x, np.ones(2))


def singular_matrix_failure1_test():
    solver = InsufficientRankSolver(epsilon=1e-4)
    assert_raises(RuntimeError, solver.solve, A, b)

def singular_matrix_failure2_test():
    solver = InsufficientRankSolver(force=True)
    assert_raises(RuntimeError, solver.solve, A, b)



from numpy.testing import assert_array_equal, assert_array_almost_equal
from numpy.testing import assert_almost_equal
from nose.tools import assert_raises, assert_true, assert_equal, assert_false
from rankit.manager import RankMerger, RankManager, RankComparer
import numpy as np
import pandas as pd


r1 = pd.DataFrame({'title': pd.Series(['A', 'B', 'C']),
                   'rate': pd.Series([1, 2, 4]),
                   'rank': pd.Series([1, 2, 3], dtype='int32')})
r2 = pd.DataFrame({'title': pd.Series(['B', 'C', 'A']),
                   'rate': pd.Series([5, 4, 3]),
                   'rank': pd.Series([1, 2, 3], dtype='int32')})


def test_rank_manager():
    mgr = RankManager()
    assert_true(mgr.cnt==0)
    mgr.update('method1', r1)
    mgr.update('method2', r2)
    assert_true(mgr.cnt==2)
    rst = mgr.get('method1')
    assert_array_equal(rst.sort_values(by='method1').loc[:, 'title'].values,
                       np.array(['A', 'B', 'C']))
    rst = mgr.get('method2')
    assert_array_equal(rst.sort_values(by='method2').loc[:, 'title'].values,
                       np.array(['B', 'C', 'A']))
    assert_true(mgr.cnt==2)
    mgr.delete('method1')
    assert_true(mgr.cnt==1)
    assert_raises(KeyError, mgr.get, 'method1')

def test_complete_rank_comparer():
    rankset = dict({'method1': r1, 'method2': r2})
    cp = RankComparer(rankset)
    assert_equal(cp.KendallMeasure('method1', 'method2'), -1.0/3.0)
    assert_equal(cp.SpearmanMeasure('method1', 'method2'), 3.5)

def test_measure_matrix():
    rankset = dict({'method1': r1, 'method2': r2})
    cp = RankComparer(rankset)
    M = cp.KendallMeasureMatrix()
    assert_true(np.min(M)>=-1.0)
    assert_true(np.max(M)<=1.0)
    M = cp.SpearmanMeaureMatrix()
    assert_true(np.min(M)>=0.0)

def test_borda_count_rank_merger():
    rankset = dict({'method1': r1, 'method2': r2})
    mgr = RankMerger(rankset)
    rst = mgr.BordaCountMerge()
    assert_array_equal(rst.loc[:, 'title'].values, np.array(['B', 'A', 'C']))

def test_avarage_rank_merger():
    rankset = dict({'method1': r1, 'method2': r2})
    mgr = RankMerger(rankset)
    rst = mgr.AverageRankMerge()
    assert_array_equal(rst.loc[:, 'title'].values, np.array(['B', 'A', 'C']))



from setuptools import setup, find_packages
from setuptools.extension import Extension
import numpy

import os

if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

with open('requirements.txt') as f:
    required = f.read().splitlines()

ext_modules=[
  Extension("rankit/util/fast_converter",
            sources = ["rankit/util/fast_converter.c"],
            include_dirs=[numpy.get_include()],
            libraries=["m"]
  ),
  Extension("rankit/manager/fast_list_matrix",
            sources = ["rankit/manager/fast_list_matrix.c"],
            include_dirs=[numpy.get_include()],
            libraries=["m"]
  )
]

setup(name="rankit",
      version="0.1.0",
      packages=find_packages(exclude=['example']),
      install_requires=required,
      include_package_data=True,
      description="A linear algebra based ranking solution",
      author="Ronnie Wang",
      author_email="geniusxiaoguai@gmail.com",
      url="http://github.com/wattlebird/ranking",
      license="MIT",
      use_2to3=True,
      ext_modules=ext_modules,
      classifiers=['Intended Audience :: Science/Research',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved',
                   'Programming Language :: Python',
                   'Topic :: Scientific/Engineering',
                   'Operating System :: Unix',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',],
      test_suite = 'nose.collector'
      )



from BaseRank import BaseRank
from ColleyRank import ColleyRank, MasseyRank
from EigRank import MarkovRank, ODRank, KeenerRank
from LeastViolatedRank import LeastViolatedRank
from DifferenceRank import DifferenceRank

__all__ = ['BaseRank', 'ColleyRank', 'MasseyRank', 'MarkovRank', 'ODRank',
'LeastViolatedRank', 'KeenerRank', 'DifferenceRank']
